@@ -137,6 +137,7 @@ struct GSLockPlugPrivate
 
         guint        idle_id;
         guint        password_check_idle_id;
+        guint        password_reset_idle_id;
         guint        response_idle_id;
 
         GTimeVal     start_time;
@@ -165,7 +166,7 @@ enum {
 static GObjectClass *parent_class = NULL;
 static guint         lock_plug_signals [LAST_SIGNAL];
 
-G_DEFINE_TYPE (GSLockPlug, gs_lock_plug, GTK_TYPE_PLUG);
+G_DEFINE_TYPE (GSLockPlug, gs_lock_plug, GTK_TYPE_PLUG)
 
 static void
 gs_lock_plug_style_set (GtkWidget *widget,
@@ -221,8 +222,9 @@ do_user_switch (GSLockPlug  *plug,
 }
 
 enum {
-        DISPLAY_NAME_COLUMN,
-        NAME_COLUMN,
+        REAL_NAME_COLUMN,
+        USER_NAME_COLUMN,
+        DISPLAY_LABEL_COLUMN,
         ACTIVE_COLUMN,
         PIXBUF_COLUMN,
         N_COLUMNS
@@ -243,7 +245,7 @@ switch_user_response (GSLockPlug *plug)
         gtk_tree_selection_get_selected (selection,
                                          &model,
                                          &iter);
-        gtk_tree_model_get (model, &iter, NAME_COLUMN, &name, -1);
+        gtk_tree_model_get (model, &iter, USER_NAME_COLUMN, &name, -1);
         if (name
             && strcmp (name, "__new_user") != 0
             && strcmp (name, "__separator") != 0) {
@@ -275,6 +277,42 @@ set_dialog_sensitive (GSLockPlug *plug,
         gtk_widget_set_sensitive (plug->action_area, sensitive);
 }
 
+static void
+remove_monitor_idle (GSLockPlug *plug)
+{
+        if (plug->priv->idle_id > 0) {
+                g_source_remove (plug->priv->idle_id);
+                plug->priv->idle_id = 0;
+        }
+}
+
+static void
+remove_password_check_idle (GSLockPlug *plug)
+{
+        if (plug->priv->password_check_idle_id > 0) {
+                g_source_remove (plug->priv->password_check_idle_id);
+                plug->priv->password_check_idle_id = 0;
+        }
+}
+
+static void
+remove_password_reset_idle (GSLockPlug *plug)
+{
+        if (plug->priv->password_reset_idle_id > 0) {
+                g_source_remove (plug->priv->password_reset_idle_id);
+                plug->priv->password_reset_idle_id = 0;
+        }
+}
+
+static void
+remove_response_idle (GSLockPlug *plug)
+{
+        if (plug->priv->response_idle_id > 0) {
+                g_source_remove (plug->priv->response_idle_id);
+                plug->priv->response_idle_id = 0;
+        }
+}
+
 static void
 gs_lock_plug_response (GSLockPlug *plug,
                        gint        response_id)
@@ -291,20 +329,10 @@ gs_lock_plug_response (GSLockPlug *plug,
                 return;
         }
 
-        if (plug->priv->idle_id > 0) {
-                g_source_remove (plug->priv->idle_id);
-                plug->priv->idle_id = 0;
-        }
-
-        if (plug->priv->password_check_idle_id > 0) {
-                g_source_remove (plug->priv->password_check_idle_id);
-                plug->priv->password_check_idle_id = 0;
-        }
-
-        if (plug->priv->response_idle_id > 0) {
-                g_source_remove (plug->priv->response_idle_id);
-                plug->priv->response_idle_id = 0;
-        }
+        remove_monitor_idle (plug);
+        remove_password_check_idle (plug);
+        remove_password_reset_idle (plug);
+        remove_response_idle (plug);
 
         if (response_id == GS_LOCK_PLUG_RESPONSE_CANCEL) {
                 gtk_entry_set_text (GTK_ENTRY (plug->priv->password_entry), "");
@@ -313,7 +341,7 @@ gs_lock_plug_response (GSLockPlug *plug,
         if (response_id == GS_LOCK_PLUG_RESPONSE_OK) {
                 gint current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (plug->priv->notebook));
 
-                if (current_page == 0) {
+                if (current_page == AUTH_PAGE) {
                         set_dialog_sensitive (plug, FALSE);
                         set_status_text (plug, _("Checking password..."));
 
@@ -349,7 +377,6 @@ monitor_progress (GSLockPlug *plug)
         GTimeVal now;
         glong    elapsed;
         glong    remaining;
-        char    *message;
 
         g_get_current_time (&now);
 
@@ -358,15 +385,17 @@ monitor_progress (GSLockPlug *plug)
         remaining = plug->priv->timeout - elapsed;
 
         if ((remaining <= 0) || (remaining > plug->priv->timeout)) {
-                message = g_strdup (_("Time has expired."));
                 set_dialog_sensitive (plug, FALSE);
-                set_status_text (plug, message);
-                g_free (message);
+                set_status_text (plug, _("Time has expired."));
 
-                if (plug->priv->response_idle_id == 0)
-                        plug->priv->response_idle_id = g_timeout_add (2000,
-                                                                      (GSourceFunc)response_idle_cb,
-                                                                      plug);
+                if (plug->priv->response_idle_id != 0)
+                        g_warning ("Response idle ID already set but shouldn't be");
+
+                remove_response_idle (plug);
+
+                plug->priv->response_idle_id = g_timeout_add (2000,
+                                                              (GSourceFunc)response_idle_cb,
+                                                              plug);
                 return FALSE;
         }
         return TRUE;
@@ -406,10 +435,7 @@ is_capslock_on (void)
 static void
 restart_monitor_progress (GSLockPlug *plug)
 {
-        if (plug->priv->idle_id > 0) {
-                g_source_remove (plug->priv->idle_id);
-                plug->priv->idle_id = 0;
-        }
+        remove_monitor_idle (plug);
 
         g_get_current_time (&plug->priv->start_time);
         plug->priv->idle_id = g_timeout_add (50,
@@ -446,15 +472,22 @@ static void
 gs_lock_plug_size_request (GtkWidget      *widget,
                            GtkRequisition *requisition)
 {
+        int mod_width;
+        int mod_height;
+
         if (GTK_WIDGET_CLASS (parent_class)->size_request)
                 GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 
-        /* Force width to be at least 1.2 times the height.
-         * This ensures enough space for the text entry and
-         * makes for a more pleasing aspect ratio */
-        if (requisition->width < requisition->height) {
-                requisition->width = requisition->height * 1.2;
+        mod_width = requisition->height * 1.3;
+        mod_height = requisition->width / 1.6;
+        if (requisition->width < mod_width) {
+                /* if the dialog is tall fill out the width */
+                requisition->width = mod_width;
+        } else if (requisition->height < mod_height) {
+                /* if the dialog is wide fill out the height */
+                requisition->height = mod_height;
         }
+
 }
 
 static void
@@ -612,6 +645,20 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                                                G_PARAM_READWRITE));
 }
 
+static gboolean
+password_reset_idle_cb (GSLockPlug *plug)
+{
+        gtk_entry_set_text (GTK_ENTRY (plug->priv->password_entry), "");
+        set_status_text (plug, "");
+        set_dialog_sensitive (plug, TRUE);
+
+        restart_monitor_progress (plug);
+
+        gtk_widget_grab_focus (plug->priv->password_entry);
+
+        return FALSE;
+}
+
 static gboolean
 password_check_idle_cb (GSLockPlug *plug)
 {
@@ -635,10 +682,11 @@ password_check_idle_cb (GSLockPlug *plug)
                                0,
                                GS_LOCK_PLUG_RESPONSE_OK);
         } else {
-                if (plug->priv->response_idle_id == 0)
-                        plug->priv->response_idle_id = g_timeout_add (4000,
-                                                                      (GSourceFunc)response_idle_cb,
-                                                                      plug);
+                remove_password_reset_idle (plug);
+
+                plug->priv->password_reset_idle_id = g_timeout_add (3000,
+                                                                    (GSourceFunc)password_reset_idle_cb,
+                                                                    plug);
 
                 set_status_text (plug, _("That password was incorrect."));
 
@@ -950,6 +998,18 @@ typedef struct
         GtkWidget  *tree;
 } DisplayChangedData;
 
+static char *
+get_user_display_label (FusaUser *user)
+{
+        char *label;
+
+        label = g_strdup_printf ("<big>%s</big>\n<small>%s</small>", 
+                                 fusa_user_get_display_name (user),
+                                 fusa_user_get_user_name (user));
+
+        return label;
+}
+
 static void
 user_displays_changed_cb (FusaUser           *user,
                           DisplayChangedData *data)
@@ -961,21 +1021,25 @@ user_displays_changed_cb (FusaUser           *user,
         int           icon_size = FACE_ICON_SIZE;
         GtkTreeModel *filter_model;
         GtkTreeModel *model;
+        char         *label;
 
         name = fusa_user_get_user_name (user);
         n_displays = fusa_user_get_n_displays (user);
         is_active = n_displays > 0;
-        pixbuf = fusa_user_render_icon (user, data->tree, icon_size, is_active);
+        pixbuf = fusa_user_render_icon (user, data->tree, icon_size);
+        label = get_user_display_label (user);
 
         filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (data->tree));
         model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
 
         gtk_list_store_set (GTK_LIST_STORE (model), &data->iter,
-                            NAME_COLUMN, name,
-                            DISPLAY_NAME_COLUMN, fusa_user_get_display_name (user),
+                            USER_NAME_COLUMN, name,
+                            REAL_NAME_COLUMN, fusa_user_get_display_name (user),
+                            DISPLAY_LABEL_COLUMN, label,
                             ACTIVE_COLUMN, is_active,
                             PIXBUF_COLUMN, pixbuf,
                             -1);
+        g_free (label);
 }
 
 static void
@@ -997,23 +1061,8 @@ populate_model (GSLockPlug   *plug,
         
         pixbuf = gtk_icon_theme_load_icon (theme, "gdm", icon_size, 0, NULL);
 
-#if 0
-        gtk_list_store_append (store, &iter);
-        gtk_list_store_set (store, &iter,
-                            DISPLAY_NAME_COLUMN, _("Log in as a new user"),
-                            NAME_COLUMN, "__new_user",
-                            PIXBUF_COLUMN, pixbuf,
-                            -1);
-        gtk_list_store_append (store, &iter);
-        gtk_list_store_set (store, &iter,
-                            DISPLAY_NAME_COLUMN, NULL,
-                            NAME_COLUMN, "__separator",
-                            -1);
-#endif
-
         profile_start ("start", "FUSA list users");
         if (! plug->priv->fusa_manager) {
-        /* for fast user switching */
                 profile_start ("start", "g_thread_init");
                 g_thread_init (NULL);
                 profile_end ("end", "g_thread_init");
@@ -1034,6 +1083,7 @@ populate_model (GSLockPlug   *plug,
                 gboolean            is_active;
                 guint               n_displays;
                 DisplayChangedData *ddata;
+                char               *label;
 
                 user = users->data;
 
@@ -1046,18 +1096,21 @@ populate_model (GSLockPlug   *plug,
                 n_displays = fusa_user_get_n_displays (user);
                 is_active = n_displays > 0;
 
-                /* this requires the following to scale well:
-                   http://bugzilla.gnome.org/show_bug.cgi?id=310418 */
-                pixbuf = fusa_user_render_icon (user, plug->priv->user_treeview, icon_size, is_active);
+                pixbuf = fusa_user_render_icon (user, plug->priv->user_treeview, icon_size);
+
+                label = get_user_display_label (user);
 
                 gtk_list_store_append (store, &iter);
                 gtk_list_store_set (store, &iter,
-                                    NAME_COLUMN, fusa_user_get_user_name (user),
-                                    DISPLAY_NAME_COLUMN, fusa_user_get_display_name (user),
+                                    USER_NAME_COLUMN, fusa_user_get_user_name (user),
+                                    REAL_NAME_COLUMN, fusa_user_get_display_name (user),
+                                    DISPLAY_LABEL_COLUMN, label,
                                     ACTIVE_COLUMN, is_active,
                                     PIXBUF_COLUMN, pixbuf,
                                     -1);
 
+                g_free (label);
+
                 ddata = g_new0 (DisplayChangedData, 1);
                 ddata->iter = iter;
                 ddata->tree = plug->priv->user_treeview;
@@ -1084,10 +1137,10 @@ compare_users (GtkTreeModel *model,
         char *label_b;
         int   result;
 
-        gtk_tree_model_get (model, a, NAME_COLUMN, &name_a, -1);
-        gtk_tree_model_get (model, b, NAME_COLUMN, &name_b, -1);
-        gtk_tree_model_get (model, a, DISPLAY_NAME_COLUMN, &label_a, -1);
-        gtk_tree_model_get (model, b, DISPLAY_NAME_COLUMN, &label_b, -1);
+        gtk_tree_model_get (model, a, USER_NAME_COLUMN, &name_a, -1);
+        gtk_tree_model_get (model, b, USER_NAME_COLUMN, &name_b, -1);
+        gtk_tree_model_get (model, a, REAL_NAME_COLUMN, &label_a, -1);
+        gtk_tree_model_get (model, b, REAL_NAME_COLUMN, &label_b, -1);
 
         if (! name_a)
                 return 1;
@@ -1146,7 +1199,7 @@ filter_out_users (GtkTreeModel *model,
         char    *name;
 
         gtk_tree_model_get (model, iter,
-                            NAME_COLUMN, &name,
+                            USER_NAME_COLUMN, &name,
                             ACTIVE_COLUMN, &is_active,
                             -1);
         if (! name)
@@ -1181,6 +1234,7 @@ setup_treeview (GSLockPlug *plug)
         profile_start ("start", NULL);
 
         store = gtk_list_store_new (N_COLUMNS,
+                                    G_TYPE_STRING,
                                     G_TYPE_STRING,
                                     G_TYPE_STRING,
                                     G_TYPE_BOOLEAN,
@@ -1208,13 +1262,13 @@ setup_treeview (GSLockPlug *plug)
 
         renderer = gtk_cell_renderer_text_new ();
         column = gtk_tree_view_column_new_with_attributes ("Name", renderer,
-                                                           "text", DISPLAY_NAME_COLUMN,
+                                                           "markup", DISPLAY_LABEL_COLUMN,
                                                            NULL);
         gtk_tree_view_append_column (GTK_TREE_VIEW (plug->priv->user_treeview), column);
 
         gtk_tree_view_set_row_separator_func (GTK_TREE_VIEW (plug->priv->user_treeview),
                                               separator_func,
-                                              GINT_TO_POINTER (NAME_COLUMN),
+                                              GINT_TO_POINTER (USER_NAME_COLUMN),
                                               NULL);
 
         gtk_tree_view_column_set_sort_column_id (column, 0);
@@ -1250,65 +1304,6 @@ get_user_display_name (void)
         return name;
 }
 
-static void
-label_set_big_bold (GtkLabel *label)
-{
-        PangoAttrList        *pattrlist;
-        PangoAttribute       *attr;
-
-        pattrlist = pango_attr_list_new ();
-
-        attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        attr = pango_attr_scale_new (1.2);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        gtk_label_set_attributes (label, pattrlist);
-
-        pango_attr_list_unref (pattrlist);
-}
-
-static void
-label_set_bigger (GtkLabel *label)
-{
-        PangoAttrList        *pattrlist;
-        PangoAttribute       *attr;
-
-        pattrlist = pango_attr_list_new ();
-
-        attr = pango_attr_scale_new (1.4);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        gtk_label_set_attributes (label, pattrlist);
-
-        pango_attr_list_unref (pattrlist);
-}
-
-static void
-label_set_big (GtkLabel *label)
-{
-        PangoAttrList        *pattrlist;
-        PangoAttribute       *attr;
-
-        pattrlist = pango_attr_list_new ();
-
-        attr = pango_attr_scale_new (1.2);
-        attr->start_index = 0;
-        attr->end_index = G_MAXINT;
-        pango_attr_list_insert (pattrlist, attr);
-
-        gtk_label_set_attributes (label, pattrlist);
-
-        pango_attr_list_unref (pattrlist);
-}
-
 static gboolean
 check_user_file (const gchar *filename,
 		 uid_t        user,
@@ -1353,7 +1348,6 @@ get_face_image ()
 {
         GtkWidget    *image;
         GdkPixbuf    *pixbuf;
-        GtkIconTheme *theme;
         const char   *homedir;
         char         *path;
         int           icon_size = 96;
@@ -1373,23 +1367,8 @@ get_face_image ()
                                                            NULL);
         }
 
-	theme = gtk_icon_theme_get_default ();
-
-        if (! pixbuf) {
-                pixbuf = gtk_icon_theme_load_icon (theme,
-                                                   "stock_person",
-                                                   icon_size,
-                                                   0,
-                                                   NULL);
-        }
-
-        if (! pixbuf) {
-                pixbuf = gtk_icon_theme_load_icon (theme,
-                                                   GTK_STOCK_MISSING_IMAGE,
-                                                   icon_size,
-                                                   0,
-                                                   NULL);
-        }
+        if (! pixbuf)
+                return NULL;
 
         image = gtk_image_new_from_pixbuf (pixbuf);
 
@@ -1417,40 +1396,33 @@ create_page_one (GSLockPlug *plug)
         vbox = gtk_vbox_new (FALSE, 12);
         gtk_container_add (GTK_CONTAINER (align), vbox);
 
-        /* should we make this a gconf preference? */
-        if (0) {
-                char *str;
-
-                /* translators: %s is a computer hostname */
-                str = g_strdup_printf (_("Welcome to %s"), g_get_host_name ());
-                widget = gtk_label_new (str);
-                gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
-                gtk_misc_set_alignment (GTK_MISC (widget), 0.5, 0.5);
-                g_free (str);
-
-                label_set_big_bold (GTK_LABEL (widget));
-        }
-
         widget = get_face_image ();
-        gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
+        if (! widget) {
+                /* placeholder */
+                widget = gtk_label_new (NULL);
+        }
+        gtk_box_pack_start (GTK_BOX (vbox), widget, TRUE, TRUE, 0);
+        gtk_misc_set_alignment (GTK_MISC (widget), 0.5, 1.0);
 
         vbox2 = gtk_vbox_new (FALSE, 0);
         gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
 
-        plug->priv->username_label = gtk_label_new (get_user_display_name ());
-        gtk_misc_set_alignment (GTK_MISC (plug->priv->username_label), 0.5, 0.5);
-        gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->username_label, FALSE, FALSE, 0);
-
-        label_set_bigger (GTK_LABEL (plug->priv->username_label));
-
-        str = g_strdup_printf ("(%s)", g_get_user_name ());
+        str = g_strdup_printf ("<span size=\"x-large\">%s</span>", get_user_display_name ());
         widget = gtk_label_new (str);
         g_free (str);
         gtk_misc_set_alignment (GTK_MISC (widget), 0.5, 0.5);
+        gtk_label_set_use_markup (GTK_LABEL (widget), TRUE);
         gtk_box_pack_start (GTK_BOX (vbox2), widget, FALSE, FALSE, 0);
 
+        str = g_strdup_printf ("<span size=\"small\">%s</span>", g_get_user_name ());
+        plug->priv->username_label = gtk_label_new (str);
+        g_free (str);
+        gtk_misc_set_alignment (GTK_MISC (plug->priv->username_label), 0.5, 0.5);
+        gtk_label_set_use_markup (GTK_LABEL (plug->priv->username_label), TRUE);
+        gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->username_label, FALSE, FALSE, 0);
+
         vbox2 = gtk_vbox_new (FALSE, 0);
-        gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox), vbox2, TRUE, TRUE, 0);
 
         hbox = gtk_hbox_new (FALSE, 6);
         gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 0);
@@ -1480,10 +1452,40 @@ create_page_one (GSLockPlug *plug)
         profile_end ("end", "page one");
 }
 
+static void
+constrain_list_size (GtkWidget      *widget,
+                     GtkRequisition *requisition,
+                     GSLockPlug     *plug)
+{
+        GtkRequisition req;
+        int            max_height;
+        int            page;
+
+        /* don't do anything if we are on the auth page */
+        page = gtk_notebook_get_current_page (GTK_NOTEBOOK (plug->priv->notebook));
+        if (page == AUTH_PAGE)
+                return;
+
+        /* constrain height to be the tree height up to a max */
+        max_height = (gdk_screen_get_height (gtk_widget_get_screen (widget))) / 4;
+        gtk_widget_size_request (plug->priv->user_treeview, &req);
+
+        requisition->height = MIN (req.height, max_height);
+}
+
+static void
+setup_list_size_constraint (GtkWidget  *widget,
+                            GSLockPlug *plug)
+{
+        g_signal_connect (widget, "size-request",
+                          G_CALLBACK (constrain_list_size), plug);
+}
+
 static void
 create_page_two (GSLockPlug *plug)
 {
-        GtkWidget            *widget;
+        GtkWidget            *header_label;
+        GtkWidget            *userlist_scroller;
         GtkWidget            *vbox;
 
         profile_start ("start", "page two");
@@ -1491,23 +1493,25 @@ create_page_two (GSLockPlug *plug)
         vbox = gtk_vbox_new (FALSE, 6);
         gtk_notebook_append_page (GTK_NOTEBOOK (plug->priv->notebook), vbox, NULL);
 
-        widget = gtk_label_new (_("Switch to user:"));
-        gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
-        gtk_box_pack_start (GTK_BOX (vbox), widget, FALSE, FALSE, 0);
+        header_label = gtk_label_new_with_mnemonic (_("S_witch to user:"));
+        gtk_misc_set_alignment (GTK_MISC (header_label), 0, 0.5);
+        gtk_box_pack_start (GTK_BOX (vbox), header_label, FALSE, FALSE, 0);
 
-        label_set_big (GTK_LABEL (widget));
-
-        widget = gtk_scrolled_window_new (NULL, NULL);
-        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (widget),
+        userlist_scroller = gtk_scrolled_window_new (NULL, NULL);
+        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (userlist_scroller),
                                              GTK_SHADOW_IN);
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (widget),
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (userlist_scroller),
                                         GTK_POLICY_NEVER,
                                         GTK_POLICY_AUTOMATIC);
-        gtk_box_pack_start (GTK_BOX (vbox), widget, TRUE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox), userlist_scroller, TRUE, TRUE, 0);
+
+        setup_list_size_constraint (userlist_scroller, plug);
 
         plug->priv->user_treeview = gtk_tree_view_new ();
         gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (plug->priv->user_treeview), FALSE);
-        gtk_container_add (GTK_CONTAINER (widget), plug->priv->user_treeview);
+        gtk_container_add (GTK_CONTAINER (userlist_scroller), plug->priv->user_treeview);
+
+        gtk_label_set_mnemonic_widget (GTK_LABEL (header_label), plug->priv->user_treeview);
 
         g_idle_add ((GSourceFunc)setup_treeview_idle, plug);
 
@@ -1590,7 +1594,7 @@ gs_lock_plug_init (GSLockPlug *plug)
         plug->priv->notebook = gtk_notebook_new ();
         gtk_notebook_set_show_tabs (GTK_NOTEBOOK (plug->priv->notebook), FALSE);
         gtk_notebook_set_show_border (GTK_NOTEBOOK (plug->priv->notebook), FALSE);
-        gtk_box_pack_start (GTK_BOX (plug->vbox), plug->priv->notebook, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (plug->vbox), plug->priv->notebook, TRUE, TRUE, 0);
 
         /* Page 1 */
 
@@ -1647,20 +1651,10 @@ gs_lock_plug_finalize (GObject *object)
         if (plug->priv->fusa_manager)
                 g_object_unref (plug->priv->fusa_manager);
 
-        if (plug->priv->password_check_idle_id > 0) {
-                g_source_remove (plug->priv->password_check_idle_id);
-                plug->priv->password_check_idle_id = 0;
-        }
-
-        if (plug->priv->response_idle_id > 0) {
-                g_source_remove (plug->priv->response_idle_id);
-                plug->priv->response_idle_id = 0;
-        }
-
-        if (plug->priv->idle_id > 0) {
-                g_source_remove (plug->priv->idle_id);
-                plug->priv->idle_id = 0;
-        }
+        remove_password_check_idle (plug);
+        remove_password_reset_idle (plug);
+        remove_response_idle (plug);
+        remove_monitor_idle (plug);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
